/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  DynamicPermissions
  Package: policy
  File:    DynamicPermissions.java
*/

package fi.hut.tcm.policy;
import java.security.*;
import java.util.Enumeration;

/**
  DynamicPermissions
  
  DynamicPermissions is a PermissionCollection that does not contain a 
  permanent set of Permissions. The permissions are fetched dynamically 
  from the Policy instead. 
*/
class DynamicPermissions
  extends PermissionCollection {

  /**
   */
  private CodeSource codesource;

  public DynamicPermissions(CodeSource cs) {
  //+ DynamicPermissions::DynamicPermissions(CodeSource)
    super();
    codesource = cs;
  //- DynamicPermissions::DynamicPermissions(CodeSource)
  }

  /**
  */
  public void add(Permission permission) {
  //+ DynamicPermissions::add(Permission)
    if (isReadOnly())
      return;
    else {
      // What should we do???
      return;
    }
  //- DynamicPermissions::add(Permission)
  }
  
  /**
  */
  public Enumeration elements() {
  //+ DynamicPermissions::elements( )
    return null;
  //- DynamicPermissions::elements( )
  }
  
  /**
  */
  public boolean implies(Permission permission) {
  //+ DynamicPermissions::implies(Permission)
    Policy p = Policy.getPolicy();
    if (p instanceof SPKIPolicy) 
      return ((SPKIPolicy)p).checkPermission(permission, codesource);
    else {
      System.out.println("Cannot check permissions: policy is no longer instance of SPKIPolicy");
      return false;
    }
  //- DynamicPermissions::implies(Permission)
  }
  
  /**
  */
  public String toString() {
  //+ DynamicPermissions::toString( )
    return new String("No permanent permissions.");
  //- DynamicPermissions::toString( )
  }
  
} // DynamicPermissions

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
