/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  CertTable
  Package: repository
  File:    CertTable.java
*/

package fi.hut.tcm.repository;
import java.util.Hashtable;
import java.util.Vector;
import fi.hut.tcm.spki.SPKICertificate;
import java.io.*;

/**
  CertTable
  
  A CertTable is a class  that stores SPKICertificates in an  indexed form that
   can be searched efficiently. Unlike Hashtable, CertTable can store 
  multiple values (SPKICertificates) per search key. 
*/
class CertTable
  extends Object
  implements Serializable {

  /**
    The certificate vectors are stored in the hashtable.
  */
  private Hashtable hash;
  
  /**
    Creates a new CertTable.
  */
  public CertTable() {
  
  //+ CertTable::CertTable( )
    hash = new Hashtable();
  //- CertTable::CertTable( )
  }
  
  /**
    Checks if the key already exists in the hashtable, and if so, adds the 
    certificate to the vector. If the key is new, creates a new vector and 
    adds it to the hashtable.
  */
  public void put(Object key, SPKICertificate cert) {
  
  //+ CertTable::put(Object, SPKICertificate)
    Vector v = (Vector) hash.get(key);
    if (v == null)
      v = new Vector();
    v.add(cert);
    hash.put(key, v);
  //- CertTable::put(Object, SPKICertificate)
  }
  
  /**
    Gets the vector that is identified by the key.
  */
  public Vector get(Object key) {
  
  //+ CertTable::get(Object)
    return (Vector) hash.get(key);
  //- CertTable::get(Object)
  }
  
  /**
    Removes the certificate from the vector identified by the key.
  */
  public void remove(Object key, SPKICertificate cert) {
  
  //+ CertTable::remove(Object, SPKICertificate)
    Vector v = (Vector) hash.get(key);
    if (v != null)
      v.removeElement(cert);
    if (!v.isEmpty())
      hash.put(key, v);
    else
      hash.remove(key);
  //- CertTable::remove(Object, SPKICertificate)
  }
  
  /**
    Implements the Serializable interface.
  */
  private void writeObject(ObjectOutputStream out)
    throws IOException  {
  
  //+ CertTable::writeObject(ObjectOutputStream)
      out.defaultWriteObject();
  //- CertTable::writeObject(ObjectOutputStream)
  }
  
  /**
    Implements the Serializable interface.
  */
  private void readObject(ObjectInputStream in)
    throws IOException, ClassNotFoundException  {
  
  //+ CertTable::readObject(ObjectInputStream)
      in.defaultReadObject();
  //- CertTable::readObject(ObjectInputStream)
  }
  
} // CertTable

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
