/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  SPKICertificateRepository
  Package: repository
  File:    SPKICertificateRepository.java
*/

package fi.hut.tcm.repository;
import java.security.cert.Certificate;
import java.util.Vector;
import java.security.*;
import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.impl.*;

/**
  SPKICertificateRepository
  
  The SPKICertificateRepository is an abstract class for storing and 
  retrieving SPKICertificates.
*/
public abstract class SPKICertificateRepository
  extends CertificateRepository {

  /**
  */
  private static SPKICertificateRepository repository;
  
  /**
  */
  protected SPKICertificateRepository() {

  //+ SPKICertificateRepository::SPKICertificateRepository()
    super("SPKI");
  //- SPKICertificateRepository::SPKICertificateRepository()
  }

  /**
  */
  public abstract Vector getCertsByIssuer(Issuer issuer);
  
  /**
  */
  public abstract Vector getCertsBySubject(Subject subject);
  
  /**
    Returns an instance of the implementation specified in the security 
    configuration file.
  */
  public static SPKICertificateRepository getInstance()
    throws InstantiationException, ClassNotFoundException, IllegalAccessException  {
  
  //+ SPKICertificateRepository::getInstance( )
    if (repository == null) {
      
      String repositoryClass = null;
      
      repositoryClass = (String) AccessController.doPrivileged(new PrivilegedAction() {
	public Object run() {
	  return Security.getProperty("certstorage.provider");
	}
      });
      
      if (repositoryClass == null) {
	repositoryClass = "fi.hut.tcm.repository.LocalSPKIRepository";
      }
      
      repository = (SPKICertificateRepository) Class.forName(repositoryClass).newInstance();
    }
    
    return repository;
  //- SPKICertificateRepository::getInstance( )
  }
  
  /**
  */
  public Vector getCertsByIssuer(Object issuer)
    throws Exception  {
  
  //+ SPKICertificateRepository::getCertsByIssuer(Object)
      if (issuer instanceof Issuer)
	return getCertsByIssuer((Issuer)issuer);
      else
	throw new Exception("Unknown issuer type" + issuer);
  //- SPKICertificateRepository::getCertsByIssuer(Object)
  }
  
  /**
  */
  public Vector getCertsBySubject(Object subject)
    throws Exception  {
  
  //+ SPKICertificateRepository::getCertsBySubject(Object)
      if (subject instanceof Subject)
	return getCertsBySubject((Subject)subject);
      else
	throw new Exception("Unknown subject type" + subject);
  //- SPKICertificateRepository::getCertsBySubject(Object)
  }
  
} // SPKICertificateRepository

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
