/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  Deleg
  Package: impl
  File:    Deleg.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;

/**
  Deleg
  
  Indicates whether authority can be forwarded to a third party. Can have 
  the value of "t" or "f".
*/
public class Deleg
  extends ByteString {

  /**
    Creates a delegation object with the default value
    "no delegation".
  */
  public Deleg() {
  
  //+ Deleg::Deleg( )
    super();
  //- Deleg::Deleg( )
  }
  
  /**
    Creates a delegation object with the boolean value.
  */
  public Deleg(boolean value) {
  
  //+ Deleg::Deleg(boolean)
    super();
    try {
      if (value)
	setValue("t");
      else
	setValue("f");
    } catch (SPKIException e) {
      System.err.println(e); // Never reached.
    }
  //- Deleg::Deleg(boolean)
  }
  
  /**
  */
  public String toString() {
  
  //+ Deleg::toString( )
    return getValueString();
  //- Deleg::toString( )
  }
  
  /**
  */
  public boolean equals(Object other) {
  
  //+ Deleg::equals(Object)
    if ((other instanceof Deleg) &&
	(((Deleg) other).getValueString().equals(this.getValueString())) &&
	(((Deleg) other).getDisplayHintString().equals(this.getDisplayHintString())))
      return true;
    else
      return false;
  //- Deleg::equals(Object)
  }
  
  /**
    Sets the value of the Deleg.
    
    Throws a SPKIException if trying to set an illegal value.
  */
  public void setValue(byte[] string)
    throws SPKIException  {
  
  //+ ByteString::setValue(byte[])
      String test = new String(string);
      if (test.equals("t") || test.equals("f"))
	super.setValue(string);
      else
	throw new SPKIException("Illegal value for deleg: " + string);
  //- ByteString::setValue(byte[])
  }
  
  /**
    Sets the value of the Deleg.
    
    Throws a SPKIException if trying to set an illegal value.
  */
  public void setValue(String string)
    throws SPKIException  {
  
  //+ ByteString::setValue(String)
      if (string.equals("t") || string.equals("f"))
	super.setValue(string);
      else
	throw new SPKIException("Illegal value for deleg: " + string);
  //- ByteString::setValue(String)
  }
  
  /**
  */
  public void accept(Visitor visitor) {
  
  //+ Deleg::accept(Visitor)
      visitor.at(this);
  //- Deleg::accept(Visitor)
  }
  
} // Deleg

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
