/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  RangeOrdering
  Package: impl
  File:    RangeOrdering.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;

/**
  RangeOrdering
  
  Defines the type of ordering in range. Allowed values are "alpha", "
  numeric", "time", "binary" and "date".
*/
class RangeOrdering
  extends ByteString {

  /**
    Creates a RangeOrdering object with the default value "alpha".
  */
  public RangeOrdering() {
  
  //+ RangeOrdering::RangeOrdering( )
    super();
    try {
      super.setValue("alpha");
    } catch (Exception e) {
      System.err.println(e.toString()); // Never reached.
    }
  //- RangeOrdering::RangeOrdering( )
  }
  
  /**
    Creates a RangeOrdering object with the string value. Allowed values 
    are "alpha", "numeric", "time", "binary" and "date".
  */
  public RangeOrdering(String value) {
  
  //+ RangeOrdering::RangeOrdering(String)
    super();
    try {
      if (value.equals("alpha") || 
	  value.equals("numeric") || 
	  value.equals("time") || 
	  value.equals("binary") || 
	  value.equals("date"))
	super.setValue(value);
      else
	super.setValue("alpha");
    } catch (Exception e) {
      System.err.println(e.toString()); // Never reached.
    }
  //- RangeOrdering::RangeOrdering(String)
  }
  
  /**
  */
  public String toString() {
  
  //+ RangeOrdering::toString( )
    return super.toString();
  //- RangeOrdering::toString( )
  }
  
  /**
  */
  public boolean equals(Object other) {
  
  //+ RangeOrdering::equals(Object)
    if ((other instanceof RangeOrdering) && 
	(((RangeOrdering)other).getValueString().equals(getValueString())) && 
	(((RangeOrdering)other).getDisplayHintString().equals(getDisplayHintString())))
      return true;
    else 
      return false;
  //- RangeOrdering::equals(Object)
  }
  
  /**
  */
  public void setValue(byte[] string)
    throws SPKIException  {
  
  //+ RangeOrdering::setValue(byte[])
    String value = new String(string);
    if (value.equals("alpha") || 
	value.equals("numeric") || 
	value.equals("time") || 
	value.equals("binary") || 
	value.equals("date"))
      super.setValue(value);
    else
      throw new SPKIException("Illegal value " + string);
  //- RangeOrdering::setValue(byte[])
  }
  
  /**
  */
  public void setValue(String string)
    throws SPKIException  {
  
  //+ RangeOrdering::setValue(String)
    if (string.equals("alpha") || 
	string.equals("numeric") || 
	string.equals("time") || 
	string.equals("binary") || 
	string.equals("date"))
      super.setValue(string);
    else
      throw new SPKIException("Illegal value " + string);
  //- RangeOrdering::setValue(String)
  }
  
  /**
  */
  public void accept(Visitor visitor) {
  
  //+ RangeOrdering::accept(Visitor)
      visitor.at(this);
  //- RangeOrdering::accept(Visitor)
  }
  
} // RangeOrdering

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
