/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  SpkiCert
  Package: impl
  File:    SpkiCert.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.format.CanonicalSpki;
import fi.hut.tcm.spki.format.CertData;
import fi.hut.tcm.spki.format.Sequence;
import java.net.URL;
import java.security.*;
import java.security.cert.*;
import java.math.BigInteger;
import java.io.IOException;
import java.io.*;

/**
  SpkiCert
  
  Our implementation of the SPKICertificate functionality.
*/
class SpkiCert
  extends SPKICertificate implements Serializable {

  public static final long serialVersionUID = 98234597;

  /**
  */
  private CertData certData;
  
  /**
  */
  private Sig signature;
    //  private Signature signature;
  
  /**
  */
  protected SpkiCert() {
  
  //+ SpkiCert::SpkiCert( )
    super();
    certData = new CertData(new Cert());
    signature = new Sig();
  //- SpkiCert::SpkiCert( )
  }
  
  /**
  */
  public boolean equals(Object other) {
  
  //+ SpkiCert::equals(Object)
    // FIX!
    if ((other instanceof SpkiCert) && other.toString().equals(this.toString()))
      return true;
    else
      return false;
  //- SpkiCert::equals(Object)
  }
  
  /**
  */
  public byte[] getEncoded() {
  
  //+ SpkiCert::getEncoded( )
    try {
      Cert c = (Cert) certData.getFormat(new Cert());
      Sequence seq = new Sequence();
      seq.add(c);
      seq.add(signature);
      CertData tmp = new CertData(seq);
      tmp.makeCanonicalSpki();
      
      return ((CanonicalSpki)tmp.getFormat(new CanonicalSpki())).getBuffer();
    } catch (Exception e) {
      System.err.println("Exception: " + e.toString());
      return null;
    }
  //- SpkiCert::getEncoded( )
  }
  
  /**
  */
  public PublicKey getPublicKey() {
  
  //+ SpkiCert::getPublicKey( )
    try {
      return certData.getIssuer();
    } catch (Exception e) {
      return null; // Issuer not a public key.
    }
  //- SpkiCert::getPublicKey( )
  }
  
  /**
  */
  public int hashCode() {
  
  //+ SpkiCert::hashCode( )
    return (new BigInteger(((ByteString) signature.get(new HashValue())).getValueBytes())).intValue();
  //- SpkiCert::hashCode( )
  }
  
  /**
  */
  public String toString() {
  
  //+ SpkiCert::toString( )
    try {
      Cert c = (Cert) certData.getFormat(new Cert());
      Sequence seq = new Sequence();
      seq.add(c);
      seq.add(signature);
      return seq.toString();
    } catch (Exception e) {
      System.err.println("Exception: " + e.toString());
      return null;
    }
  //- SpkiCert::toString( )
  }
  
  /**
  */
  public void sign(KeyPair key) {
  
  //+ SpkiCert::sign(KeyPair)
    MessageDigest md;
    Signature sign;
    byte[] hashBytes;
    byte[] sigBytes; 
    byte[] rawData;
    try {
      sign = Signature.getInstance("SHA-1/DSA");
      sign.initSign(key.getPrivate());
      md = MessageDigest.getInstance("SHA");

      certData.makeCanonicalSpki();
      rawData = ((CanonicalSpki)certData.getFormat(new CanonicalSpki())).getBuffer();
      md.update(rawData);
      hashBytes = md.digest();
      sign.update(rawData);
      sigBytes = sign.sign();
      
      Hash hash = new Hash();
      HashAlgName algname = new HashAlgName();
      algname.add(new ByteString("sha1"));
      hash.add(algname);
      hash.add(new HashValue(hashBytes));
      PubKey principal = new PubKey(key.getPublic());
      SigVal sigval = new SigVal();
      sigval.add(new ByteString(sigBytes));
      
      signature.add(hash);
      signature.add(principal);
      signature.add(sigval);
    } catch (Exception e) {
      System.err.println("Exception: " + e.toString());
    }
  //- SpkiCert::sign(KeyPair)
  }
  
  /**
  */
  public void verify(PublicKey key) 
       throws CertificateException, 
	 NoSuchAlgorithmException,
	 InvalidKeyException,
	 NoSuchProviderException,
	 SignatureException  {
  
  //+ SpkiCert::verify(PublicKey)
    sun.misc.CharacterDecoder decoder64 = new sun.misc.BASE64Decoder();

    String halgname;
    MessageDigest md;
    String salgname;
    Signature sign;
    String sigStr;
    byte[] hashBytes;
    byte[] sigBytes; 
    byte[] rawData;

    try {
      halgname = ((ByteString) ((Composite) ((Composite) signature.get(0)).get(0)).get(0)).getValueString();
      salgname = ((ByteString) ((Composite) ((Composite) signature.get(1)).get(0)).get(0)).getValueString();

      if (halgname.equals("sha1")) {
	md = MessageDigest.getInstance("SHA");
      } else {
	throw new NoSuchAlgorithmException(halgname);
      }
      if (salgname.equals("dsa-sha1")) {
	sign = Signature.getInstance("SHA-1/DSA");
	sign.initVerify(key);
      } else {
	throw new NoSuchAlgorithmException(salgname);
      }
      
      // get the hash of the data of this certificate (in certData)
    
      certData.makeCanonicalSpki();
      rawData = ((CanonicalSpki)certData.getFormat(new CanonicalSpki())).getBuffer();
      //      hashBytes = md.digest(rawData);
      certData.parseCanonicalSpki();

      // get the signature bytes from the signature and verify them
      sigStr = ((ByteString) ((Composite) signature.get(2)).get(0)).getValueString();
      if (sigStr.startsWith("|") && sigStr.endsWith("|")) {
	// BASE64 encoded
	sigBytes = decoder64.decodeBuffer(sigStr.substring(1, sigStr.length() - 1));
      } else {
	System.out.println("no BASE64 encoding");
	sigBytes = sigStr.getBytes();
      }

    } catch (IOException e) {
      throw new CertificateException("Parse error");
    } catch (NoSuchFieldException e) {
      throw new CertificateException("Parse error");
    }

    // verify
    //    sign.update(hashBytes);
    sign.update(rawData);
    sign.verify(sigBytes);

    // XXX
    try {
      getIssuer();
    } catch (Exception e) {
      System.out.println("verify leaves the SPKICertificate in an unacceptable state");
    }

  //- SpkiCert::verify(PublicKey)
  }
  
  /**
  */
  public void verify(PublicKey key, String sigProvider) 
    throws  CertificateException, NoSuchAlgorithmException,
      InvalidKeyException, NoSuchProviderException,
      SignatureException {
  
  //+ SpkiCert::verify(PublicKey, String)
    sun.misc.CharacterDecoder decoder64 = new sun.misc.BASE64Decoder();

    String halgname;
    MessageDigest md;
    String salgname;
    Signature sign;
    String sigStr;
    byte[] hashBytes;
    byte[] sigBytes; 
    byte[] rawData;

    try {
      halgname = ((ByteString) ((Composite) ((Composite) signature.get(0)).get(0)).get(0)).getValueString();
      salgname = ((ByteString) ((Composite) ((Composite) signature.get(1)).get(0)).get(0)).getValueString();

      if (halgname.equals("sha1")) {
	md = MessageDigest.getInstance("SHA");
      } else {
	throw new NoSuchAlgorithmException(halgname);
      }
      if (salgname.equals("dsa-sha1")) {
	sign = Signature.getInstance("SHA-1/DSA", sigProvider);
	sign.initVerify(key);
      } else {
	throw new NoSuchAlgorithmException(salgname);
      }
      
      // get the hash of the data of this certificate (in certData)
    
      certData.makeCanonicalSpki();
      rawData = ((CanonicalSpki)certData.getFormat(new CanonicalSpki())).getBuffer();
      //      hashBytes = md.digest(rawData);
      certData.parseCanonicalSpki();

      // get the signature bytes from the signature and verify them
      sigStr = ((ByteString) ((Composite) signature.get(2)).get(0)).getValueString();
      if (sigStr.startsWith("|") && sigStr.endsWith("|")) {
	// BASE64 encoded
	sigBytes = decoder64.decodeBuffer(sigStr.substring(1, sigStr.length() - 1));
      } else {
	System.out.println("no BASE64 encoding");
	sigBytes = sigStr.getBytes();
      }

    } catch (IOException e) {
      throw new CertificateException("Parse error");
    } catch (NoSuchFieldException e) {
      throw new CertificateException("Parse error");
    }

    // verify
    //    sign.update(hashBytes);
    sign.update(rawData);
    sign.verify(sigBytes);

    // XXX
    try {
      getIssuer();
    } catch (Exception e) {
      System.out.println("verify leaves the SPKICertificate in an unacceptable state");
    }

  //- SpkiCert::verify(PublicKey, String)
  }
  
  /**
    Returns the certificate version number. 
    
  */
  public int getVersion() {
  
  //+ SpkiCert::getVersion( )
    return 0;
  //- SpkiCert::getVersion( )
  }
  
  /**
    Sets the data and signature according to the sequence.
  */
  public void putSequence(Sequence seq) {
  
  //+ SpkiCert::putSequence(Sequence)
	certData = new CertData((Cert) seq.get(0));
	signature = (Sig) seq.get(1);
  //- SpkiCert::putSequence(Sequence)
  }

  /**
    Sets a display hint for how the SPKI certificate  
    is best displayed. The hint is optional and may be ignored.
    
    The verifier of the certificate should treat the CertDisplay field as a
     comment, i.e. ignore it.
  */
  protected void putCertDisplay(ByteString certDisplay) 
    throws NoSuchFieldException,NoSuchMethodException  {
  
  //+ SpkiCert::putCertDisplay(ByteString)
    certData.putCertDisplay((CertDisplay) certDisplay);
  //- SpkiCert::putCertDisplay(ByteString)
  }
  
  /**
    Gets the display hint for how the SPKI certificate  
    is best displayed. The hint is optional and may be ignored.
    
    The verifier of the certificate should treat the CertDisplay field as a
     comment, i.e. ignore it.
  */
  public ByteString getCertDisplay() 
    throws NoSuchFieldException,NoSuchMethodException  {
  
  //+ SpkiCert::getCertDisplay( )
    return ((ByteString) certData.getCertDisplay());
  //- SpkiCert::getCertDisplay( )
  }
  
  /**
  */
  protected void putIssuer(PublicKey issuer)
    throws IllegalStateException,NoSuchMethodException  {
  
  //+ SpkiCert::putIssuer(PublicKey)
      certData.putIssuer(issuer);
  //- SpkiCert::putIssuer(PublicKey)
  }
  
  /**
  */
  public PublicKey getIssuer()
    throws NoSuchFieldException,NoSuchMethodException  {
  
  //+ SpkiCert::getIssuer( )
    return certData.getIssuer();
  //- SpkiCert::getIssuer( )
  }
  
  /**
  */
  protected void putIssuerInfo(URL[] info) 
    throws NoSuchFieldException,NoSuchMethodException {
  
  //+ SpkiCert::putIssuerInfo(URL[])
      certData.putIssuerInfo(info);
  //- SpkiCert::putIssuerInfo(URL[])
  }
  
  /**
  */
  public URL[] getIssuerInfo() 
    throws NoSuchFieldException,NoSuchMethodException {
  
  //+ SpkiCert::getIssuerInfo( )
    return certData.getIssuerInfo();
  //- SpkiCert::getIssuerInfo( )
  }
  
  /**
  */
  protected void putSubject(Object subject)
    throws IllegalStateException,NoSuchMethodException  {
  
  //+ SpkiCert::putSubject(Object)
      certData.putSubject(subject);
  //- SpkiCert::putSubject(Object)
  }
  
  /**
  */
  public Object getSubject()
    throws NoSuchFieldException,NoSuchMethodException  {
  
  //+ SpkiCert::getSubject( )
    return certData.getSubject();
  //- SpkiCert::getSubject( )
  }
  
  /**
  */
  protected void putSubjectInfo(URL[] info)
    throws NoSuchFieldException,NoSuchMethodException {
  
  //+ SpkiCert::putSubjectInfo(URL[])
      certData.putSubjectInfo(info);
  //- SpkiCert::putSubjectInfo(URL[])
  }
  
  /**
  */
  public URL[] getSubjectInfo()
    throws NoSuchFieldException,NoSuchMethodException {
  
  //+ SpkiCert::getSubjectInfo( )
    return certData.getSubjectInfo();
  //- SpkiCert::getSubjectInfo( )
  }
  
  /**
  */
  protected void putDeleg(boolean deleg)
    throws IllegalStateException,NoSuchMethodException  {
  
  //+ SpkiCert::putDeleg(boolean)
      certData.putDeleg(new Deleg(deleg));
  //- SpkiCert::putDeleg(boolean)
  }
  
  /**
  */
  public boolean getDeleg()
    throws NoSuchFieldException,NoSuchMethodException  {
  
  //+ SpkiCert::getDeleg( )
      if (certData.getDeleg().getValueString().equals("t"))
	return true;
      else 
	return false;
  //- SpkiCert::getDeleg( )
  }
  
  /**
  */
  protected void putValidity(Validity valid)
    throws IllegalStateException,NoSuchMethodException  {
  
  //+ SpkiCert::putValidity(Validity)
      certData.putValidity(valid);
  //- SpkiCert::putValidity(Validity)
  }
  
  /**
  */
  public Validity getValidity()
    throws NoSuchFieldException,NoSuchMethodException  {
  
  //+ SpkiCert::getValidity( )
    return certData.getValidity();
  //- SpkiCert::getValidity( )
  }
  
  /**
  */
  protected void putTag(fi.hut.tcm.spki.Tag tag)
    throws IllegalStateException,NoSuchMethodException  {
  
  //+ SpkiCert::putTag(Tag)
      certData.putTag((fi.hut.tcm.spki.impl.Tag) tag);
  //- SpkiCert::putTag(Tag)
  }
  
  /**
  */
  public fi.hut.tcm.spki.Tag getTag()
    throws NoSuchFieldException,NoSuchMethodException  {
  
  //+ SpkiCert::getTag( )
    return (fi.hut.tcm.spki.Tag) certData.getTag();
  //- SpkiCert::getTag( )
  }
  
  /**
  */
  protected void putComment(ByteString comment)
    throws IllegalStateException,NoSuchMethodException  {
  
  //+ SpkiCert::putComment(ByteString)
      certData.putComment((Comment) comment);
  //- SpkiCert::putComment(ByteString)
  }
  
  /**
  */
  public ByteString getComment()
    throws NoSuchFieldException,NoSuchMethodException  {
  
  //+ SpkiCert::getComment( )
    return (ByteString) certData.getComment();
  //- SpkiCert::getComment( )
  }
  
  /**
  */
  public boolean isValid() {
  
  //+ SpkiCert::isValid( )
    try {
      Validity v = certData.getValidity();
      if (v == null)
	return true;
      else
	return (v.test());
    } catch (Exception e) {
      return false;
    }
  //- SpkiCert::isValid( )
  }
  
  /**
  */
  private void writeObject(java.io.ObjectOutputStream out)
    throws IOException {

    //+ SpkiCert::writeObject(ObjectOutputStream)
      try {
	Cert c = (Cert) certData.getFormat(new Cert());
	Sequence seq = new Sequence();
	seq.add(c);
	seq.add(signature);
	CertData tmp = new CertData(seq);
	tmp.makeCanonicalSpki();
	byte[] buf = ((CanonicalSpki)tmp.getFormat(new CanonicalSpki())).getBuffer();
	out.writeObject(buf);

      } catch (Exception e) {
	// FIX: do something that makes sense!
	throw new IOException(e.getMessage());
      }
    //- SpkiCert::writeObject(ObjectOutputStream)
  }

  /**
  */
  private void readObject(java.io.ObjectInputStream in)
    throws IOException, ClassNotFoundException {

    //+ SpkiCert::readObject(ObjectInputStream)
      try {
	byte[] buf = (byte[]) in.readObject();
	CanonicalSpki canonical = new CanonicalSpki();
	canonical.putBuffer(buf);
	CertData tmp = new CertData(canonical);
	tmp.parseCanonicalSpki();

	Sequence seq = (Sequence) tmp.getFormat(new Sequence());
	certData = new CertData((Cert) seq.get(0));
	signature = (Sig) seq.get(1);

      } catch (ClassNotFoundException e) {
	throw e;
      } catch (Exception e) {
	// FIX: do something that makes sense!
	//	e.printStackTrace();
	throw new IOException(e.getMessage());
      }
    //- SpkiCert::readObject(ObjectInputStream)
  }

} // SpkiCert

// +*************** UNUSED CODE ***************
//+ SpkiCert::getValidity( )
//- SpkiCert::getValidity( )
//+ SpkiCert::getSubject( )
//- SpkiCert::getSubject( )
//+ SpkiCert::putTag(Tag)
//- SpkiCert::putTag(Tag)
//+ SpkiCert::putDeleg(Boolean)
//- SpkiCert::putDeleg(Boolean)
//+ SpkiCert::getIssuer( )
//- SpkiCert::getIssuer( )
//+ SpkiCert::isValid( )
//- SpkiCert::isValid( )
//+ SpkiCert::getTag( )
//- SpkiCert::getTag( )
//+ SpkiCert::putIssuer(PublicKey)
//- SpkiCert::putIssuer(PublicKey)
//+ SpkiCert::putValidity(Validity)
//- SpkiCert::putValidity(Validity)
//+ SpkiCert::putComment(ByteString)
//- SpkiCert::putComment(ByteString)
//+ SpkiCert::putSubject(Object)
//- SpkiCert::putSubject(Object)
//+ SpkiCert::getDeleg( )
//- SpkiCert::getDeleg( )
//+ SpkiCert::getComment( )
//- SpkiCert::getComment( )
//+ SpkiCert::getValidity( )
//- SpkiCert::getValidity( )
//+ SpkiCert::getSubject( )
//- SpkiCert::getSubject( )
//+ SpkiCert::putTag(Tag)
//- SpkiCert::putTag(Tag)
//+ SpkiCert::putDeleg(Boolean)
//- SpkiCert::putDeleg(Boolean)
//+ SpkiCert::getIssuer( )
//- SpkiCert::getIssuer( )
//+ SpkiCert::isValid( )
//- SpkiCert::isValid( )
//+ SpkiCert::getTag( )
//- SpkiCert::getTag( )
//+ SpkiCert::putIssuer(PublicKey)
//- SpkiCert::putIssuer(PublicKey)
//+ SpkiCert::putValidity(Validity)
//- SpkiCert::putValidity(Validity)
//+ SpkiCert::putComment(ByteString)
//- SpkiCert::putComment(ByteString)
//+ SpkiCert::putSubject(Object)
//- SpkiCert::putSubject(Object)
//+ SpkiCert::getDeleg( )
//- SpkiCert::getDeleg( )
//+ SpkiCert::getComment( )
//- SpkiCert::getComment( )
//+ SpkiCert::getValidity( )
//- SpkiCert::getValidity( )
//+ SpkiCert::getSubject( )
//- SpkiCert::getSubject( )
//+ SpkiCert::putTag(Tag)
//- SpkiCert::putTag(Tag)
//+ SpkiCert::putDeleg(Boolean)
//- SpkiCert::putDeleg(Boolean)
//+ SpkiCert::getIssuer( )
//- SpkiCert::getIssuer( )
//+ SpkiCert::isValid( )
//- SpkiCert::isValid( )
//+ SpkiCert::getTag( )
//- SpkiCert::getTag( )
//+ SpkiCert::putIssuer(PublicKey)
//- SpkiCert::putIssuer(PublicKey)
//+ SpkiCert::putValidity(Validity)
//- SpkiCert::putValidity(Validity)
//+ SpkiCert::putComment(ByteString)
//- SpkiCert::putComment(ByteString)
//+ SpkiCert::putSubject(Object)
//- SpkiCert::putSubject(Object)
//+ SpkiCert::getDeleg( )
//- SpkiCert::getDeleg( )
//+ SpkiCert::getComment( )
//- SpkiCert::getComment( )
// -*************** UNUSED CODE ***************
