/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  SpkiCrl
  Package: impl
  File:    SpkiCrl.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;
import java.util.*;
import java.security.cert.Certificate;

/**
  SpkiCrl
  
  
*/
class SpkiCrl
  extends SPKICRL
  implements fi.hut.tcm.spki.Composite, Component {

  /**
  */
  private Vector children;
  
  /**
  */
  protected SpkiCrl() {
  
  //+ SpkiCrl::SpkiCrl( )
    super();
    children = new Vector();
  //- SpkiCrl::SpkiCrl( )
  }
  
  /**
  */
  public boolean equals(Object other) {
  
  //+ SpkiCrl::equals(Object)
    return ((other instanceof SPKICRL) &&
	    ((new String(((SPKICRL) other).getEncoded())).equals(new String(this.getEncoded()))));
  //- SpkiCrl::equals(Object)
  }
  
  /**
  */
  public byte[] getEncoded() {
  
  //+ SpkiCrl::getEncoded( )
    return null;
  //- SpkiCrl::getEncoded( )
  }
  
  /**
  */
  public int getVersion() {
  
  //+ SpkiCrl::getVersion( )
    return 0;
  //- SpkiCrl::getVersion( )
  }
  
  /**
  */
  public Date getNotBefore() {
  
  //+ SpkiCrl::getNotBefore( )
    NotBefore nb = (NotBefore) get(new NotBefore());
    if (nb != null) {
      Time t = (Time) nb.get(new Time());
      if (t != null)
	return t.getDate();
      else 
	return null;
    } else
      return null;
  //- SpkiCrl::getNotBefore( )
  }
  
  /**
  */
  public Date getNotAfter() {
  
  //+ SpkiCrl::getNotAfter( )
    NotAfter na = (NotAfter) get(new NotAfter());
    if (na != null) {
      Time t = (Time) na.get(new Time());
      if (t != null)
	return t.getDate();
      else 
	return null;
    } else
      return null;
  //- SpkiCrl::getNotAfter( )
  }
  
  /**
  */
  public boolean isValid() {
  
  //+ SpkiCrl::isValid( )
    Date nb = getNotBefore();
    Date na = getNotAfter();
    Date now = new Date();
    if ((nb != null) && now.before(nb)) 
      return false;
    else if ((na != null) && (now.after(na)))
      return false;
    else
      return true;
  //- SpkiCrl::isValid( )
  }
  
  /**
  */
  public boolean isRevoked(Certificate cert) {
  
  //+ SpkiCrl::isRevoked(Certificate)
    Iterator i = children.iterator();
    return false;
  //- SpkiCrl::isRevoked(Certificate)
  }
  
  /**
  */
  public String toString() {
  
  //+ SpkiCrl::toString( )
    String result = new String("(crl (version 0) " + 
			       get(new HashList()).toString());
    if (getNotBefore() != null)
      result = new String(result + get(new NotBefore()).toString());
    if (getNotAfter() != null)
      result = new String(result + get(new NotAfter()).toString());
    result = new String(result + ")");
    return result;
  //- SpkiCrl::toString( )
  }
  
  /**
    Add a new component to a the crl. Returns the index of the added 
    component. This index number may later change if earlier components are
    removed.
  */
  public int add(Component addThis)
    throws IllegalArgumentException,IllegalStateException  {
  
  //+ SpkiCrl::add(Component)
      if (check(addThis)) {
	children.addElement(addThis);
	return children.size() - 1;
      }
      else
	throw new IllegalArgumentException("Check failed: " + addThis);
  //- SpkiCrl::add(Component)
  }

  /**
    Get returns the wanted child by it's number. If not found, throws 
    ArrayIndexOutOfBoundsException.
  */
  public Component get(int number)
    throws ArrayIndexOutOfBoundsException  {
  
  //+ SpkiCrl::get(int)
      return (Component) children.elementAt(number);
  //- SpkiCrl::get(int)
  }
  
  /**
    This function returns the first child x that fullfills (x instanceof 
    childClass). This is handy for searching Composites and subclasses of 
    ByteString, but not ByteStrings.
  */
  public Component get(Component childClass) {
  
  //+ SpkiCrl::get(Component)
      for (int i = 0; i < children.size(); i++)
	if (childClass.getClass().isInstance(children.elementAt(i)))
	  return (Component) children.elementAt(i);
      return null;
  //- SpkiCrl::get(Component)
  }

  /**
    Removes the first occurrence of this object. Returns true if the 
    argument was a component of this composite; false otherwise. Elements 
    of a higher index are moved one step downwards.
  */
  public boolean remove(Component child) {
  
  //+ SpkiCrl::remove(Component)
    return children.removeElement(child);
  //- SpkiCrl::remove(Component)
  }
  
  /**
    Returns an iterator over all the children.
  */
  public Iterator iterator() {
  
  //+ SpkiCrl::iterator( )
    return children.iterator();
  //- SpkiCrl::iterator( )
  }
  
  /**
  */
  public int size() {
  
  //+ SpkiCrl::size( )
    return children.size();
  //- SpkiCrl::size( )
  }
  
  /**
    Checks if the component to be added is of correct type. Returns true if
     it is, false if it is not. Throws IllegalStateException if this 
    component is of right type, but cannot be added now (there is already a
     max number of such components, for example).
  */
  public boolean check(Component addition)
    throws IllegalStateException  {
  
  //+ SpkiCrl::check(Component)
    if (addition instanceof HashList) {
      if (size() == 0)
	return true;
      else
	throw new IllegalStateException();
    }
    else if ((addition instanceof Validity) && 
	     ((size() == 1) || (size() == 2)))
      return true;
    else
      return false;
  //- SpkiCrl::check(Component)
  }
  
  /**
  */
  public void accept(Visitor visitor) {
  
  //+ Composite::accept(Visitor)
    visitor.at(this);
  //- Composite::accept(Visitor)
  }
  
} // SpkiCrl

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
