/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  Uri
  Package: impl
  File:    Uri.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;
import java.net.URL;

/**
  Uri
  
  A reference to a resource.
*/
public class Uri
  extends ByteString {

  /**
  */
  private URL url;
  
  /**
    Creates an empty uri. The value must be set before 
    the object is used. An object with NULL value will be ignored.
  */
  public Uri() {
  
  //+ Uri::Uri( )
    super();
  //- Uri::Uri( )
  }
  
  /**
  */
  public Uri(URL url) {
  
  //+ Uri::Uri(URL)
    super();
    this.url = url;
    try {
      setValue(url.toString());
    } catch (Exception e) {
      System.err.println(e); // Never reached.
    }
  //- Uri::Uri(URL)
  }
  
  /**
  */
  public Uri(String value) {
  
  //+ Uri::Uri(String)
    super();
    try {
      URL u = new URL(value);
      url = u;
      super.setValue(u.toString());
    } catch (Exception e) {
      System.err.println("Cannot create an Url with value " + value + e); 
    }
  //- Uri::Uri(String)
  }
  
  /**
  */
  public Uri(String value, String display) {
  
  //+ Uri::Uri(String, String)
    super();
    try {
      URL u = new URL(value);
      url = u;
      super.setValue(u.toString());
      super.setDisplayHint(display);
    } catch (Exception e) {
      System.err.println("Cannot create an Url with value " + value + e); 
    }
  //- Uri::Uri(String, String)
  }
  
  /**
  */
  public URL getUrl() {
  
  //+ Uri::getUrl( )
    return url;
  //- Uri::getUrl( )
  }
  
  /**
  */
  public void setUrl(URL url) {
  
  //+ Uri::setUrl(URL)
    this.url = url;
    try {
      super.setValue(url.toString());
    } catch (Exception e) {
      System.err.println(e.toString()); // Never reached.
    }
  //- Uri::setUrl(URL)
  }
  
  /**
  */
  public String toString() {
  
  //+ Uri::toString( )
    return url.toString();
  //- Uri::toString( )
  }
  
  /**
  */
  public boolean equals(Object other) {
  
  //+ Uri::equals(Object)
    if ((other instanceof Uri) &&
	(((Uri) other).getValueString().equals(this.getValueString())) &&
	(((Uri) other).getDisplayHintString().equals(this.getDisplayHintString())))
      return true;
    else
      return false;
  //- Uri::equals(Object)
  }
  
  /**
  */
  public void setValue(byte[] string)
    throws SPKIException  {
  
  //+ Uri::setValue(byte[])
    try {
      URL u = new URL(new String(string));
      url = u;
      super.setValue(string);
    } catch (Exception e) {
      throw new SPKIException(e.toString());
    }
  //- Uri::setValue(byte[])
  }
  
  /**
  */
  public void setValue(String string)
    throws SPKIException  {
  
  //+ Uri::setValue(String)
    try {
      URL u = new URL(string);
      url = u;
      super.setValue(string);
    } catch (Exception e) {
      throw new SPKIException(e.toString());
    }
  //- Uri::setValue(String)
  }
  
  /**
  */
  public void accept(Visitor visitor) {
  
  //+ Uri::accept(Visitor)
    visitor.at(this);
  //- Uri::accept(Visitor)
  }
  
} // Uri

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
