import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.impl.*;
import fi.hut.tcm.policy.*;
import fi.hut.tcm.repository.*;
import java.security.Security;
import java.security.KeyPair;
import java.security.KeyStore;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.Policy;
import java.security.CodeSource;
import java.security.PermissionCollection;
import java.security.cert.*;
import java.net.URL;
import java.io.InputStream;
import sun.security.util.PropertyExpander;
import java.util.PropertyPermission;

class testi {

  public final static void main(String[] args) throws Exception {

    System.out.println("* Installing provider...");
    Security.addProvider(new fi.hut.tcm.spki.impl.SPKIProvider());
    Provider prov = Security.getProvider("TeSSA.tcm.hut.fi SPKIProvider");
    System.out.println("Provider name: " + prov.getName());
    System.out.println("Provider version: " + prov.getVersion());
    System.out.println("Provider info: " + prov.getInfo());

    System.out.println("* Creating policy...");
    SPKIPolicy pol = new SPKIPolicy();
    System.out.println("Policy: " + pol.toString());
    
    System.out.println("* Fetching keys...");
    KeyStore ks = KeyStore.getInstance("JKS");
    URL store = new URL(PropertyExpander.expand("file:${user.home}/.keystore"));
    InputStream stream = store.openStream(); 
    ks.load(stream, "passwd".toCharArray());
    stream.close();
    Certificate c = ks.getCertificate("Self");
    KeyPair self = new KeyPair(c.getPublicKey(), (PrivateKey) ks.getKey("Self", "passwd".toCharArray()));
    PublicKey admin = ks.getCertificate("Admin").getPublicKey();
    //    System.out.println("Self public key: " + self.getPublic().toString());
    //    System.out.println("Self private key: " + self.getPrivate().toString());
    //    System.out.println("Admin key: " + admin.toString());

    System.out.println("* Creating certificates...");
    SPKICertificateFactory spookymaker = (SPKICertificateFactory) new SpkiCertFactory();
    Certificate[] certs = new Certificate[1];
    fi.hut.tcm.spki.Tag tag = new fi.hut.tcm.spki.impl.Tag();
    tag.add(new TagStar());
    //    System.out.println("Tag: " + tag.toString());
    certs[0] = spookymaker.makeCert(self, admin, false, tag);
    //    System.out.println("Certificate: " + certs[0].toString());

    System.out.println("Encoded cert: ");
    System.out.println(new String(certs[0].getEncoded()));

    URL url = new URL("http://www.nixu.fi/");
    CodeSource cs = new CodeSource(url, certs);
    System.out.println("* Getting permissions...");
    PermissionCollection perm = pol.getPermissions(cs);
    System.out.println("Permissions: " + perm.toString());

    System.out.println("* Testing permissions...");
    Policy.setPolicy(pol);
    boolean result = perm.implies(new PropertyPermission("os.name","read"));
    System.out.println("Result: " + result);
    result = perm.implies(new java.io.FilePermission("foo","read"));
    System.out.println("Result: " + result);
  }

}
